struct IpPortData {
    ip: IpAddr,
    port: u16,
}

struct DeviceStateData {
    id: Uuid,
    state: DeviceState,
}

struct DeviceIdData {
    id: Uuid,
}

struct NewConfigurationData {
    device_type_id: Uuid,
    configuration: LaunchConfiguration
}

#[post("/admin/types/device/new")] add_device_type(data: Data<Database>, device_type: Json<DeviceType>)

#[get("/admin/types/device/get_all")]get_device_types(data: Data<Database>)

#[post("/admin/types/attraction/new")]add_attraction_type(data: Data<Database>,attraction_type: Json<AttractionType>,)

#[get("/get/attractions")]get_attractions(data: Data<Database>)

#[post("/controllers/vibrate")]vibrate_controllers(data: Data<Database>, obj: Json<DeviceIdData>)

#[post("/device/state")]update_device_state(data: Data<Database>, obj: Json<DeviceStateData>)

#[post("/device/update")]update_device(data: Data<Database>, obj: Json<DeviceIdData>)

#[post("/device-type/configuraitons/add")]add_configuration(data: Data<Database>, obj: Json<NewConfigurationData>)

#[post("/extension/add")]add_extension(data: Data<Database>, obj: Json<Extension>)


struct NewDeviceRequest {
    ip: IpAddr,
    device_name: Option<String>,
    port: u16,
    device_type: Uuid,
}

struct AddDeviceToGroupData {
    device_id: String,
    group_id: String,
}

struct AddMacAddr {
    device_id: Uuid,
    mac_addr: MacAddr6,
}

struct WolDataWithIp {
    ip: IpAddr,
    mac_addr: String,
}

struct DeviceId {
    id: Uuid
}

struct DeviceStateData {
    state: DeviceState
}

#[post("/connections/delete")]delete_device(data: Data<Database>, device: Json<DeviceId>)

#[get("/connections/devices")]list_devices(data: Data<Database>)

#[get("/connections/device")]get_device_data(data: Data<Database>, req: Json<IdNameData>)

#[post("/connections/state")]update_state(data: Data<Database>, req: HttpRequest, state: Json<DeviceStateData>)

#[post("/connections/new/no-check")]connect_without_check(data: Data<Database>,req: Json<NewDeviceRequest>)

#[post("/connections/new")]connect(data: Data<Database>, req: Json<NewDeviceRequest>)

#[post("/connections/new/ssh")]connect_via_ssh(data: Data<Database>, req: Json<NewDeviceRequest>)

#[post("/connections/check")]refresh_device_connection_state(data: Data<Database>,req: Json<IdNameData>)

#[post("/connections/check/ip")]refresh_device_connection_state_by_ip(data: Data<Database>,req: HttpRequest,obj: Json<DeviceStateData>)

#[get("/connections/groups")]list_groups(data: Data<Database>)

#[get("/wol/add")]add_wol_device(data: Data<Database>, req: Json<AddMacAddr>)

#[post("/wol/wake")]wake_device_up(data: Data<Database>, req: Json<IdNameData>)

#[post("/wol/wake/ip")]wake_device_up_with_ip(req: Json<WolDataWithIp>)

#[derive(Debug, Clone)]
struct TypedDeviceData {
    device_type: DeviceType,
    devices: Vec<DeviceData>
}


#[get("/admin")]admin(data: Data<Database>)

#[get("/devices")]get_all_devices_data(data: Data<Database>)

#[get("/run/{id}")]run_app(path: web::Path<(Uuid,)>, data: Data<Database>)

#[get("/run/attraction/{id}")]run_attraction(path: web::Path<(Uuid,)>, data: Data<Database>)

#[get("/new_device")]new_device(data: Data<Database>)

#[get("/new_attraction")]new_attraction()

#[get("/run/device/with_parameters/{id}/{configuration_id}")]run_device_with_parameters(path: web::Path<(Uuid,Uuid)>, data: Data<Database>)

#[get("/device/{id}")]get_device_admin_page(path: web::Path<Uuid>, data: Data<Database>)


type DeviceArguments =(IdNameData, Vec<String>);
type DeviceEnvironment = (IdNameData, HashMap<String,String>);

struct DeviceReqData {
    identifier: IdNameData,
    configuration: IdNameData,
    args: Vec<String>,
    env: HashMap<String,String>
}

struct AttractionReqData {
    attraction: IdNameData,
    device_ids: Vec<IdNameData>,
    args: Vec<DeviceArguments>,
    environment: Vec<DeviceEnvironment>
}

struct SessionReqData {
    session_id: Uuid
}

struct SessionResData {
    uid: Uuid
}

#[post("/runner/stop/attraction")]stop_session(data: Data<Database>,session: Json<SessionReqData>)

#[post("/runner/run/device")]run_device_configuration(data: Data<Database>,device_data: Json<DeviceReqData>)

#[post("/runner/run/attraction")]run_attraction_configuration(data: Data<Database>,attraction_data: Json<AttractionReqData>)

struct SyncReq {
    callback_id: Uuid,
    session_id: Uuid
}

#[post("/runner/sync")]sync_device_configuration(req: Json<SyncReq> ,data: Data<Database>)

#[get("/health_check")]health_check()

#[get("/inner/check-for-updates/game")]
check_for_updates(data: Data<Database>)

/// Downloads the game if needed to central server
#[get("/inner/update-if-necessary/game")]
update_game_if_necessary(data: Data<Database>)


#[get("/package/{id:.*}/modified")]get_package_hash(path: web::Path<(String,)>)


#[get("/update_all")]update_all()

struct App {
    id: String,
    attraction_type: String,
    max_players: u8,
}

#[derive(Debug, Deserialize, Serialize)]
enum RemoteAppType {
    ManagementSystemServer,
    ManagementSystemAgent,
    Game,
}

#[derive(Debug, Deserialize, Serialize)]
struct RemoteApp {
    app_type: RemoteAppType,
    version: Version,
}

#[derive(Debug, Deserialize, Serialize, Clone)]
struct AttractionType {
    pub uid: Uuid,
    pub name: String,
    //Map of  devices types uuids and counts. If need just add device type to available, add (device_type, 0)
    pub minimal_necessary_devices: HashMap<Uuid, i32>,
    pub configurations: HashMap<Uuid, Uuid>,
    pub need_sync: bool,
    pub post_sync_configurations: HashMap<Uuid, Uuid>
}

struct DeviceGroup {
    uid: Uuid,
    attraction_name: String,
    device_ids: Vec<Uuid>
}

struct LaunchParameters {
    pub server_ip_port: Option<String>,
    pub session_id: Option<Uuid>,
    pub program: String,
    pub args: Vec<String>,
    pub working_dir: Option<String>,
    pub environment_vars: Option<HashMap<String,String>>,
    pub callback_link: Option<String>
}

struct Process {
    launch_parameters: LaunchParameters,
    uuid: Uuid,
    created_at: SystemTime,
}

struct SystemData{
    pub mac_addr: String,
    pub os: String,
    pub device_name: String
}

struct ControllerData{
    pub device_id: u32,
    pub charge_level: f32
}

struct ProcessData {
    pub launch_parameters: LaunchConfigurationClient,
    pub uuid: Uuid,
    pub created_at: SystemTime,
}

struct ChargeLevel {
    pub level: f32
}

struct PivotData {
    pub host: String,
    pub system_data: SystemData,
    pub controllers: Vec<ControllerData>,
    pub processes: Vec<Process>,
    pub charge_level: Option<ChargeLevel>
}

struct DeviceType {
    pub uid: Uuid,
    pub name: String,
    //Command for running
    pub configurations: Vec<LaunchConfiguration>,
    pub is_command_remote: bool,
}

enum DeviceState {
    Inactive,
    Working,
    Prepared,
    Unavailable,
}

struct DeviceData {
    pub id: Uuid,
    pub device_name: String,
    pub device_type: Uuid,
    pub device_state: DeviceState,
    pub ip: IpAddr,
    pub port: u16,
    pub mac: Option<String>
}

enum ExtensionPoint {
    TransformRequestBeforeAtraction,
    TransformConfigurationsBeforeAttraction
}

struct Extension {
    pub uid: Uuid,
    pub extension_point: ExtensionPoint,
    pub script: String
}

struct LaunchConfiguration {
    pub uid: Uuid,
    pub session_id: Option<Uuid>,
    pub name: String,
    pub program: String,
    pub args: Vec<String>,
    pub working_dir: Option<String>,
    pub environment_vars: HashMap<String,String>,
    pub need_sync: bool,
    pub after_stop_trigger: Option<Trigger>
}

struct LaunchConfigurationClient {
    pub server_ip_port: String,
    pub session_id: Option<Uuid>,
    pub program: String,
    pub args: Vec<String>,
    pub working_dir: Option<String>,
    pub environment_vars: HashMap<String,String>,
    pub callback_id: Option<String>,
    pub after_stop_trigger: Option<Trigger>
}

struct Trigger {
    pub trigger_data: String,
}

struct GitlabPackageFile {
    pub id: i32,
    pub file_name: String,
    pub created_at: DateTime<Utc>
}

struct PackageMetadata {
    pub version: Version,
    pub package_name: String
}

struct GitlabPackage {
    pub id: u32,
    pub name: String,
    pub version: Version,
    pub created_at: DateTime<Utc>
}

enum SessionState {
    Initialized,
    Working,
    Finished,
    Aborted,
    FinishedWithError
}

struct Session {
    pub session_id: Uuid,
    pub devices: Vec<Uuid>,
    pub attraction_type_id: Uuid,
    pub created_at: DateTime<Utc>,
    pub state: SessionState
}

struct SyncData {
    pub uid: Uuid,
    pub session_id: Uuid,
    pub created_at: DateTime<Utc>,
    pub client_id: Uuid
}

struct WolData {
    pub device_id: Uuid,
    pub mac_addr: MacAddr6
}