extern crate dirs;

use std::env;
use std::fs::File;
use std::io::Read;
use std::path::Path;
use local_ip_address::local_ip;
use yaml_rust::{Yaml, YamlLoader};

pub fn read_ip_port() -> String {
    let _server_ip = local_ip().unwrap();
    let _server_port = read_config_field("server-port").unwrap().as_i64().unwrap();
    return format!("{_server_ip}:{_server_port}");
}


fn read_val_from_doc<'a>(doc: Yaml, key: &str) -> Yaml {
    match doc {
        Yaml::Hash(ref h) => {
            for (k, v) in h {
                if key == k.as_str().unwrap() {
                    return v.clone();
                }
            }
            panic!("Can't find key")
        }
        _ => {
            panic!("Can't find hash from doc")
        }
    }
}

fn adjust_canonicalization<P: AsRef<Path>>(p: P) -> String {
    const VERBATIM_PREFIX: &str = r#"\\?\"#;
    let p = p.as_ref().display().to_string();
    if p.starts_with(VERBATIM_PREFIX) {
        p[VERBATIM_PREFIX.len()..].to_string()
    } else {
        p
    }
}

pub fn get_exe_path() -> String {
    let mut exe_dir = env::current_exe().unwrap();
    exe_dir.pop();
    let path = exe_dir.canonicalize().unwrap();
    adjust_canonicalization(path)
}

pub fn read_config_field<'a>(field: &str) -> Result<Yaml, String> {
    let exe_dir = get_exe_path();
    log::info!("Reading config file: {}", format!("{exe_dir}/config.yaml"));
    let config_file = File::open(format!("{exe_dir}/config.yaml"));
    if config_file.is_err() {
        return Err("Can't open file".to_string());
    }
    let mut config_str = String::new();

    let to_string_res = config_file.unwrap().read_to_string(&mut config_str);
    if to_string_res.is_err() {
        return Err("Can't convert file data to string".to_string());
    }
    let docs = YamlLoader::load_from_str(&config_str);
    if docs.is_err() {
        return Err("Can't parse data to yaml".to_string());
    }
    let document = docs.unwrap();
    let document_copy = document[0].clone();
    let elem = read_val_from_doc(document_copy, field);
    let res = Ok(elem);
    return res;
}
