#[macro_export]
macro_rules! generate_body {
    (get_all, $self:ident, $class_t:ty $(, $filter_id:ident : $filter:expr )* $(,)?) => {
        {
           let filter = crate::doc_filter!($($filter_id : $filter ),*);
            let mut cursors = $self
             .col
             .find(filter, None)
             .await
             .ok()
             .expect("Error getting data");

            let mut docs: Vec<$class_t> = Vec::new();

            while let Some(doc) = cursors
                .try_next()
                .await
                .ok()
                .expect("Error mapping through cursor")
           {
               docs.push(<$class_t>::from_document(&doc))
           }
           Ok(docs)
        }
    };
    (get, $self:ident, $class_t:ty $(, $filter_id:ident : $filter:expr )*) => {
        {
        let filter = crate::doc_filter!($($filter_id : $filter ),*);
        let mut cursors = $self
            .col
            .find(filter, None)
            .await
            .ok()
            .expect("Can't get device data");

        let doc_opt = cursors.try_next().await.ok().expect("Error mapping through cursor");
        if doc_opt.is_none() {
            return  Err("Can't unwrap device data")
        }
        let obj = <$class_t>::from_document(&doc_opt.unwrap());
        Ok(obj)
    }
    };
    (delete, $self:ident, $class_t:ty $(, $filter_id:ident : $filter:expr )*) => {
        {
        let filter = crate::doc_filter!($($filter_id : $filter ),*);
        let cursors = $self
            .col
            .delete_one(filter, None)
            .await;
        if let Ok(c) = cursors { Ok(c.deleted_count) }
        else { Err("Can't create new obj") }
    }
    };
    (delete_all, $self:ident, $class_t:ty $(, $filter_id:ident : $filter:expr )*) => {
        {
        let filter = crate::doc_filter!($($filter_id : $filter ),*);
        let cursors = $self
            .col
            .delete_many(filter, None)
            .await;
        if let Ok(c) = cursors { Ok(c.deleted_count) }
        else { Err("Can't create new obj") }
    }
    };



    (add, $self: ident, $obj:ident) => {
      {
        let res = $self
            .col
            .insert_one($obj.to_document(), None)
            .await;
        
        if let Err(e) = res { Err("Can't create new obj") }
        else { Ok($obj) }

      }
    };
    (update, $self:ident, $class_t:ty $(, $filter_id:ident : $filter:expr )*; $($update_id:ident = $update:expr),+) => {
      {
        let filter = crate::doc_filter!($($filter_id : $filter ),*);
        let update =  mongodb::bson::doc!{"$set" : {$(stringify!($update_id) : $update ),*}};
        let res = $self
            .col
            .update_one(filter, update, None)
            .await;
        if let Err(e) = res { Err("Can't create new obj") }
        else { Ok(()) }
      }
    };
    (update_all, $self:ident, $class_t:ty $(, $filter_id:ident : $filter:expr )*; $($update_id:ident = $update:expr),+) => {
      {
        let filter = crate::doc_filter!($($filter_id : $filter ),*);
        let update =  mongodb::bson::doc!{"$set" : {$(stringify!($update_id) : $update ),*}};
        let res = $self
            .col
            .update_many(filter, update, None)
            .await;
        if let Err(e) = res { Err("Can't create new obj") }
        else { Ok(()) }
      }
    };
    (get_n, $cnt:tt, $self:ident, $class_t:ty $(, $filter_id:ident : $filter:expr )* $(,)?) => {
        {
           let filter = crate::doc_filter!($($filter_id : $filter ),*);
            let mut cursors = $self
             .col
             .find(filter, None)
             .await
             .ok()
             .expect("Error getting data");

            let mut docs: Vec<$class_t> = Vec::new();
            let mut count = $cnt;
           if count <= 0 {
               return Ok(docs)
           }

            while let Some(doc) = cursors
                .try_next()
                .await
                .ok()
                .expect("Error mapping through cursor")
           {
               docs.push(<$class_t>::from_document(&doc));
               count -= 1;
               if count <= 0 {
                    return Ok(docs)
               }
           }
           Err("Can't find entities")
        }
    };
}

#[macro_export]
macro_rules! identify {
    (get_all, $class_t:ty) => {Vec<$class_t>};
    (get, $class_t:ty) => {$class_t};
    (add, $class_t:ty) => {$class_t};
    (delete, $class_t:ty) => {u64};
    (delete_all, $class_t:ty) => {u64};
    (update, $class_t:ty) => {()};
    (update_all, $class_t:ty) => {()};
    (get_n, $class_t:ty) => {Vec<$class_t>}
}


#[macro_export]
macro_rules! doc_filter {
    ($($filter_id:ident : $filter:expr ),+) => {
      mongodb::bson::doc!{"$and": [ $({stringify!($filter_id) : $filter }),*]}
    };
    ($filter_id:ident : $filter:expr $(,)?) => {
       mongodb::bson::doc!{stringify!($update_id) : $update}
    };
    ($(,)?) => {
       None
    };

}

#[macro_export]
macro_rules! crud {
    (get_n, $class_t:ty, $postfix:ident $(,$filter_id:ident : $filter_ty:ty = $filter:expr )* $(,)?) => {
        ::paste::paste! {
            #[allow(non_snake_case)]
            pub async fn [<get _ n _ $class_t _ $postfix>](&self, cnt: i32, $($filter_id : $filter_ty),*) -> Result<crate::identify!(get_n, $class_t), &str>
                {
                    crate::generate_body!(get_n, cnt, self, $class_t  $(, $filter_id : $filter )*)
                }

        }
    };
    (add, $class_t:ty) => {
        ::paste::paste! {
            #[allow(non_snake_case)]
            pub async fn [<add _ $class_t>](&self, it : $class_t) -> Result<crate::identify!(add, $class_t), &str>
                {
                    crate::generate_body!(add, self, it)
                }

        }
    };
    ($req: ident, $class_t:ty, $postfix:ident $(,$filter_id:ident : $filter_ty:ty = $filter:expr )* ; $($update_id:ident : $update_ty:ty = $update:expr),+ $(,)?) => {
        ::paste::paste! {
            #[allow(non_snake_case)]
            pub async fn [<$req _ $class_t _ $postfix>](&self $(, $filter_id : $filter_ty)+ $(, $update_id : $update_ty)+) -> Result<crate::identify!($req, $class_t), &str>
                {
                    crate::generate_body!($req, self, $class_t  $(, $filter_id : $filter )*; $($update_id = $update),+ )
                }
        }
    };
    ($req:ident, $class_t:ty) => {
        ::paste::paste! {
           #[allow(non_snake_case)] 
            pub async fn [<$req _ $class_t>](&self) -> Result<crate::identify!($req, $class_t), &str>
                {
                    crate::generate_body!($req, self, $class_t)
                }

        }
    };
    ($req:ident, $class_t:ty, $postfix:ident $(,$filter_id:ident : $filter_ty:ty = $filter:expr )+ $(,)?) => {
        ::paste::paste! {
            #[allow(non_snake_case)]
            pub async fn [<$req _ $class_t _ $postfix>](&self, $($filter_id : $filter_ty),*) -> Result<crate::identify!($req, $class_t), &str>
                {
                    crate::generate_body!($req, self, $class_t  $(, $filter_id : $filter )*)
                }

        }
    };

    (get, $cnt:tt, $class_t:ty, $postfix:ident $(,$filter_id:ident : $filter_ty:ty = $filter:expr )* $(,)?) => {
        ::paste::paste! {
            #[allow(non_snake_case)]
            pub async fn [<get _ $class_t _ $cnt _ $postfix>](&self, $($filter_id : $filter_ty),*) -> Result<crate::identify!(get_n, $class_t), &str>
                {
                    crate::generate_body!(get_n, $cnt, self, $class_t  $(, $filter_id : $filter )*)
                }

        }
    };
}

mod test {
    use bson::Document;
    use mongodb::Collection;
    use crate::auxiliary::bson_serializable::BsonSerializable;
    use serde::{Serialize, Deserialize};
    use futures::TryStreamExt;


    #[derive(Debug, Deserialize, Serialize)]
    struct Test{
        x: i32,
        y: i32
    }
    impl BsonSerializable for Test{}

    #[derive(Debug, Clone)]
    struct TestCol{
        col: Collection<Document>
    }

    impl TestCol {
        crud!(get_all, Test, by_x, x: &str="1",);
        crud!(get_all, Test);
        crud!(get, Test);
        crud!(get, 2, Test , by_x_with_y, z: &str="1");
        crud!(get_n, Test, by_x_with_y, x: &str="1");
        crud!(add, Test);
        crud!(get_all, Test, by_x_with_y, x: &str="1");
        crud!(update, Test, by_x_with, x: &str="1" ; theta: i32 = theta );
        crud!(update_all, Test, by_x_with, x: &str="1" ; theta: i32 = theta, zeta:i32  = zeta );
        crud!(delete, Test, by_x, x:&str="1");
        crud!(delete_all, Test, by_x, x:&str="1");
    }

    impl Test {
        fn test(){
        }
    }
}
