use uuid::Uuid;
use serde::{Deserialize, Serialize};


pub fn validate_find_by_name_result<T: Clone>(data: Vec<T>) -> Result<T, String> {
    if data.len() == 1 {
        return Ok(data[0].clone())
    }
    Err("Invalid result count".to_string())
}


#[derive(Debug, Deserialize, Serialize)]
pub struct IdNameData {
    id: Option<Uuid>,
    name: Option<String>
}

impl IdNameData {
    pub fn new_name(name: &str) -> Self {
        Self {
            id: None,
            name: Some(name.to_string())
        }
    }
    pub fn new_id(id: Uuid) -> Self {
        Self {
            id: Some(id),
            name: None
        }
    }

}

pub enum IdName {
    Id(Uuid),
    Name(String),
    Error
}

impl IdNameData {
    pub fn get_type(&self) -> IdName {
        if let Some(id) = &self.id {
            return IdName::Id(id.clone())
        }
        else if let Some(name) = &self.name {
            return IdName::Name(name.clone())
        }

        IdName::Error
    }
}

