use std::collections::HashMap;
use uuid::Uuid;
use serde::{Serialize, Deserialize};
use crate::auxiliary::bson_serializable::BsonSerializable;

#[derive(Debug, Deserialize, Serialize, Clone)]
pub struct AttractionType {
   #[serde(default = "Uuid::new_v4")]
    pub uid: Uuid,
    pub name: String,
    //Map of  devices types uuids and counts. If need just add device type to available, add (device_type, 0)
    pub minimal_necessary_devices: HashMap<Uuid, i32>,
    pub configurations: HashMap<Uuid, Uuid>,
    pub need_sync: bool,
    pub post_sync_configurations: HashMap<Uuid, Uuid>
}

impl AttractionType {
    pub fn new(
        name: String, 
        minimal_necessary_devices: HashMap<Uuid, i32>, 
        configurations: HashMap<Uuid, Uuid>,
        post_sync_configurations: HashMap<Uuid, Uuid>,
        need_sync: bool) -> Self {

        Self {
            uid: Uuid::new_v4(),
            name,
            minimal_necessary_devices,
            configurations,
            need_sync,
            post_sync_configurations
        }
    }
}

impl BsonSerializable for AttractionType {}
