use mongodb::bson::{doc, Document};
use serde::{Deserialize, Serialize};
use std::str::FromStr;
use uuid::Uuid;
use crate::auxiliary::bson_serializable::BsonSerializable;
use crate::model::launch_configuration::LaunchConfiguration;

#[derive(Debug, Deserialize, Serialize, Clone)]
pub struct DeviceType {
    #[serde(default = "Uuid::new_v4")]
    pub uid: Uuid,
    pub name: String,
    //Command for running
    pub configurations: Vec<LaunchConfiguration>,
    //TODO
    pub is_command_remote: bool,
}

impl DeviceType {
    pub fn new(name: String, configurations: Vec<LaunchConfiguration>, is_command_remote: bool) -> Self {
        Self {
            uid: Uuid::new_v4(),
            name,
            configurations,
            is_command_remote,
        }
    }
}

impl DeviceType {
    pub fn from_document(doc: &Document) -> Self {
        let uid = Uuid::from_str(doc.get_str("uid").unwrap()).unwrap();
        let name = doc.get_str("name").unwrap().to_string();
        let configurations = doc.get_array("configurations").unwrap().iter()
            .map(|it| LaunchConfiguration::from_document(it.as_document().unwrap()))
            .collect();

        let is_command_remote = doc.get_bool("is_command_remote").unwrap();

        DeviceType {
            uid,
            name,
            configurations,
            is_command_remote,
        }
    }

    pub fn to_document(&self) -> Document {
        let configs: Vec<Document> = self.configurations.iter().map(|it| LaunchConfiguration::to_document(it)).collect();
        doc! {
            "uid": self.uid.to_string(),
            "name": self.name.to_string(),
            "configurations": configs,
            "is_command_remote": self.is_command_remote
        }
    }
}
