use std::{fmt, io};

use js_sandbox::Script;
use serde::{Deserialize, Serialize, de::DeserializeOwned};
use uuid::Uuid;
use crate::auxiliary::bson_serializable::BsonSerializable;


#[derive(Copy, Clone, Debug, Deserialize, Serialize, PartialEq)]
pub enum ExtensionPoint {
    TransformRequestBeforeAtraction,
    TransformConfigurationsBeforeAttraction,
    CountSessionClientsNumber
}

impl fmt::Display for ExtensionPoint {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "{:?}", self)
    }
}

#[derive(Clone, Debug, Deserialize, Serialize)]
pub struct Extension {
    #[serde(default = "Uuid::new_v4")]
    pub uid: Uuid,
    pub extension_point: ExtensionPoint,
    pub script: String
}

impl Extension {
    pub fn exec<T: DeserializeOwned+Serialize >(&self, x: &T) -> Result<T, io::Error> {
        let mut script = Script::from_string(self.script.as_str()).map_err(|x| io::Error::new(io::ErrorKind::Other ,x.to_string()))?;
        script.call("execute", (x,)).map_err(|x| io::Error::new(io::ErrorKind::Other, x.to_string()))
    }

    pub fn exec_v2<T1: DeserializeOwned + Serialize, T2: DeserializeOwned + Serialize>(&self, x: &T1) ->  Result<T2, io::Error> {
        let mut script = Script::from_string(self.script.as_str()).map_err(|x| io::Error::new(io::ErrorKind::Other ,x.to_string()))?;
        script.call("execute", (x,)).map_err(|x| io::Error::new(io::ErrorKind::Other, x.to_string()))
    }
}

impl BsonSerializable for Extension {}
