use std::str::FromStr;
use mongodb::bson::{doc, Document};
use serde::{Serialize, Deserialize};
use crate::model::version::Version;



#[derive(Debug, Serialize, Deserialize)]
pub struct PackageMetadata {
    pub version: Version,
    pub package_name: String
}

impl PackageMetadata {
    pub fn new(package_name: &str, version: Version) -> Self {
        Self{
            package_name: package_name.to_string(),
            version,
        }
    }
    pub fn from_document(doc: &Document) -> Self {
        let version = Version::from_str(doc.get("version").unwrap().as_str().unwrap()).unwrap();

        let package_name = doc.get("package_name").unwrap().as_str().unwrap();

        Self::new(
            package_name,
            version
        )
    }

    pub fn to_document(&self) -> Document {
        let it = self.clone();
        let version = &it.version;
        let package = &it.package_name;

        return doc!{
            "version": version.to_string(),
            "package_name": package
        }
    }
}