use chrono::{DateTime, Utc};
use mongodb::bson::{doc};
use serde::{Serialize, Deserialize};
use uuid::Uuid;
use crate::auxiliary::bson_serializable::BsonSerializable;

#[derive(Debug,Serialize, Deserialize, Copy, Clone)]
pub struct SyncData {
    pub uid: Uuid,
    pub session_id: Uuid,
    #[serde(with = "bson::serde_helpers::chrono_datetime_as_bson_datetime", default = "Utc::now" )]
    pub created_at: DateTime<Utc>,
    pub client_id: Uuid
}


impl SyncData {
    pub fn new(client_id: Uuid, session_id: &Uuid) -> Self {
        SyncData {
            uid: Uuid::new_v4(),
            created_at: Utc::now(),
            session_id: session_id.clone(),
            client_id
        }
    }
}

impl BsonSerializable for SyncData {}