use std::str::FromStr;
use macaddr::{MacAddr6};
use mongodb::bson::{doc, Document};
use serde::{Serialize, Deserialize};
use uuid::Uuid;

#[derive(Debug, Serialize, Deserialize)]
pub struct WolData {
    pub device_id: Uuid,
    pub mac_addr: MacAddr6
}

impl WolData {
    pub fn to_document(&self) -> Document {
        let it = self.clone();
        let id = &it.device_id;
        let mac = &it.mac_addr.to_string();
        return doc!{
            "id": id.to_string(),
            "mac": mac,
        }
    }

    pub fn from_document(doc: &Document) -> Self {
        let device_id = Uuid::parse_str(doc.get("id").unwrap().to_string().as_str()).unwrap();

        let mac_addr = MacAddr6::from_str(
            doc.get("mac").unwrap().as_str().unwrap()
        ).unwrap();

        Self {
            device_id,
            mac_addr
        }
    }
}