use crate::auxiliary::mongo_error::Error;
use crate::model::attraction_type::AttractionType;
use futures::TryStreamExt;
use mongodb::bson::{doc, Document};
use mongodb::Collection;
use uuid::Uuid;
use crate::auxiliary::bson_serializable::BsonSerializable;
use crate::crud;

#[derive(Clone, Debug)]
pub struct AttractionTypeRepo {
    col: Collection<Document>,
}

impl AttractionTypeRepo {
    pub fn new(col: Collection<Document>) -> Self {
        AttractionTypeRepo { col }
    }
}

impl AttractionTypeRepo {
   crud!(get_all, AttractionType, by_name, name: &str = name.to_string());

    pub async fn create(&self, obj: AttractionType) -> Result<AttractionType, Error> {
        let _ = self
            .col
            .insert_one(obj.to_document(), None)
            .await
            .ok()
            .expect("Can't create new user");
        Ok(obj)
    }

    pub async fn get_all(&self) -> Result<Vec<AttractionType>, Error> {
        let mut cursors = self
            .col
            .find(None, None)
            .await
            .ok()
            .expect("Error getting list_devices of devices");

        let mut objs: Vec<AttractionType> = Vec::new();

        while let Some(doc) = cursors
            .try_next()
            .await
            .ok()
            .expect("Error mapping through cursor")
        {
            objs.push(AttractionType::from_document(&doc))
        }
        Ok(objs)
    }

    pub async fn get(&self, id: &Uuid) -> Result<AttractionType, &str> {
        let filter = doc! {"uid": id.to_string()};
        let mut cursors = self
            .col
            .find(filter, None)
            .await
            .ok()
            .expect("Can't get group data");

        let doc_opt = cursors
            .try_next()
            .await
            .ok()
            .expect("Error mapping through cursor");
        if doc_opt.is_none() {
            return Err("Can't unwrap device data");
        }
        let device = AttractionType::from_document(&doc_opt.unwrap());
        Ok(device)
    }

    pub async fn exists_by_name(&self, name: &str) -> Result<bool, &str> {
        let filter = doc! {"name": name};
        let mut cursors = self
            .col
            .find(filter, None)
            .await
            .ok()
            .expect("Can't get group data");

        let doc_opt = cursors
            .try_next()
            .await
            .ok()
            .expect("Error mapping through cursor");
        if doc_opt.is_none() {
            return Ok(false);
        }
        Ok(true)
    }
}
