use std::net::IpAddr;
use futures::TryStreamExt;
use mongodb::{Collection};
use mongodb::bson::{Document};
use uuid::Uuid;
use crate::auxiliary::bson_serializable::BsonSerializable;
use crate::crud;
use crate::model::device::{DeviceData, DeviceState};

#[derive(Clone, Debug)]
pub struct DeviceRepo {
    col: Collection<Document>
}


impl DeviceRepo {
    pub fn new(col: Collection<Document>) -> Self{
        DeviceRepo{col}
    }
}

impl DeviceRepo {
    crud!(get_all, DeviceData);
    crud!(get, DeviceData, by_uid, id: &Uuid = id.to_string());
    crud!(get, DeviceData, by_ip, ip: &IpAddr = ip.to_string());
    crud!(get , DeviceData, by_device_state, device_state : DeviceState = device_state.to_string());
    crud!(get_n , DeviceData, by_device_type_and_state, device_type: &Uuid = device_type.to_string(), device_state : DeviceState = device_state.to_string());
    crud!(get_all , DeviceData, by_device_type_and_state, device_type: &Uuid = device_type.to_string(), device_state : DeviceState = device_state.to_string());
    crud!(get_n , DeviceData, by_device_type, device_type: &Uuid = device_type.to_string());
    crud!(get_all , DeviceData, by_device_type, device_type: &Uuid = device_type.to_string());
    crud!(get_n , DeviceData, by_device_state, device_state : DeviceState = device_state.to_string());
    crud!(get_all, DeviceData, by_name, device_name: &str = device_name.to_string());
    crud!(add, DeviceData);
    crud!(update, DeviceData, state_by_id, id: &Uuid = id.to_string(); device_state : DeviceState = device_state.to_string());
    crud!(update, DeviceData, mac_by_id, id: &Uuid = id.to_string(); mac : String = Some(mac));
    crud!(delete, DeviceData, by_id, id: &Uuid = id.to_string());
}
