use futures::TryStreamExt;
use mongodb::bson::{doc, Document};
use mongodb::{Collection, IndexModel};
use uuid::Uuid;
use crate::crud;
use crate::model::extension::{Extension, ExtensionPoint};
use crate::auxiliary::bson_serializable::BsonSerializable;

#[derive(Clone, Debug)]
pub struct ExtensionsRepo {
    col: Collection<Document>,
}

impl ExtensionsRepo {
    pub fn new(col: Collection<Document>) -> Self {
        ExtensionsRepo { col }
    }
}

impl ExtensionsRepo {
    crud!(add, Extension);
    crud!(get, Extension, by_uid, uid: &Uuid = uid.to_string());
    crud!(get, Extension, by_extension_point, extension_point: ExtensionPoint = extension_point.to_string());
    crud!(get_all, Extension, by_extension_point, extension_point: ExtensionPoint = extension_point.to_string());
}
