use mongodb::{Collection};
use mongodb::bson::{Document};
use crate::crud;
use crate::auxiliary::bson_serializable::BsonSerializable;
use crate::model::session::{Session, SessionState};
use futures::TryStreamExt;
use uuid::Uuid;

#[derive(Clone, Debug)]
pub struct SessionRepo {
    col: Collection<Document>
}


impl SessionRepo {
    pub fn new(col: Collection<Document>) -> Self{
        SessionRepo{col}
    }
}

impl SessionRepo {
    crud!(add, Session);
    crud!(get_all, Session);
    crud!(get, Session, by_id, session_id: &Uuid = session_id.to_string());
    crud!(delete, Session, by_id, session_id: &Uuid = session_id.to_string());
    crud!(update, Session, state_by_id, session_id: &Uuid = session_id.to_string(); state : SessionState = state.to_string());
}
