use futures::TryStreamExt;
use mongodb::bson::{doc, Document};
use mongodb::Collection;
use crate::model::package_metadata::PackageMetadata;
use crate::model::version::Version;

#[derive(Clone, Debug)]
pub struct SoftwareDataRepo {
    col: Collection<Document>
}

impl SoftwareDataRepo {
    pub fn new(col: Collection<Document>) -> Self{
        SoftwareDataRepo {col}
    }

    pub async fn get_latest_version_package(&self, package_name: &str) -> Result<PackageMetadata, &str> {
        let filter = doc!{"package_name": package_name.to_string()};
        let mut cursors = self
            .col
            .find(filter, None)
            .await
            .ok()
            .expect("Error getting latest_version of package");

        let package_doc_opt = cursors.try_next().await.ok().expect("Error mapping through cursor");
        if package_doc_opt.is_none() {
            return  Err("Can't unwrap device data")
        }
        let package = PackageMetadata::from_document(&package_doc_opt.unwrap());
        Ok(package)
    }

    pub async fn add_package_version(&self, package_name: &str, version: Version) -> Result<PackageMetadata, &str> {
        let package = PackageMetadata::new(package_name, version);
        let _package = self
            .col
            .insert_one(package.to_document(), None)
            .await
            .ok()
            .expect("Can't create new user");
        Ok(package)
    }

    pub async fn update_package_version(&self, package_name: &str, version: Version) -> Result<PackageMetadata, &str> {

        let filter = doc!{"package_name": package_name};
        let update = doc!{"$set": {"version": version.to_string()}};

        let package = PackageMetadata::new(package_name, version);

        let res = self
            .col
            .update_one(filter, update, None)
            .await;

        if res.is_ok() && res.unwrap().modified_count == 0 {
            return Self::add_package_version(self, package_name, version).await;
        }

        Ok(package)

    }

    //TODO(Replace package version)
}