use futures::TryStreamExt;
use mongodb::{Collection};
use mongodb::bson::{doc, Document};
use uuid::Uuid;
use crate::auxiliary::mongo_error::Error;
use crate::model::wol_data::WolData;

#[derive(Clone, Debug)]
pub struct WolRepo {
    col: Collection<Document>
}

impl WolRepo {
    pub fn new(col: Collection<Document>) -> Self{
        WolRepo{col}
    }
}

impl WolRepo {
    pub async fn add_new_device(&self, data: WolData) -> Result<WolData, Error> {
        let _device = self
            .col
            .insert_one(data.to_document(), None)
            .await
            .ok()
            .expect("Can't create new user");
        Ok(data)
    }

    pub async fn get_device_mac_addr(&self, device_id: &Uuid) -> Result<WolData, &str> {
        let filter = doc!{"id":device_id.to_string()};
        let mut cursors = self
            .col
            .find(filter, None)
            .await
            .ok()
            .expect("Can't get device data");

        let device_doc_opt = cursors.try_next().await.ok().expect("Error mapping through cursor");
        if device_doc_opt.is_none() {
            return  Err("Can't unwrap device data")
        }
        let device = WolData::from_document(&device_doc_opt.unwrap());
        Ok(device)
    }
}